/*
 * Created on: Jul 28, 2024
 */

#include <stdint.h>

#include "ui.h"

#include "adc.h"
#include "btn.h"
#include "config.h"
#include "led.h"
#include "ledprog_pep.h"
#include "ledprog_rgb.h"



#define MODE_RUN                0
#define MODE_PROGRAM            1
#define MODE_PARAMETER          2

#define UI_CONF_SAVE_TIMEOUT    384

#define UI_PROG_RUNTIME_MIN     (128*15)    // 15 seconds
#define UI_PROG_RUNTIME_MAX     (128*120)   // 120 seconds

#define PROG_RANDOM             0x80



static uint8_t mode = MODE_RUN;
static uint8_t tick = 0;

static uint16_t save_delay = 0;



void ui_btn_push_cb(uint8_t idx)
{

}

void ui_btn_hold_cb(uint8_t idx)
{
    /*
    switch (idx) {
        case 0: {   // left pepper hat
            userconf.pep_prog_ena_map ^= PROG_RANDOM;
            break;
        }
        case 1: {   // right pepper hat
            userconf.rgb_prog_ena_map ^= PROG_RANDOM;
            break;
        }
    }
    */
}

void ui_btn_release_cb(uint8_t idx)
{
    uint8_t update;

    switch (idx) {
        case 0: {   // left pepper hat
            update = userconf.pep_prog_ena_map & ~(PROG_RANDOM);
            update++;
            if (update > 6) update = 0;
            userconf.pep_prog_ena_map = update | (userconf.pep_prog_ena_map & PROG_RANDOM);
            ledprog_pep_init();
            break;
        }
        case 1: {   // right pepper hat
            update = userconf.rgb_prog_ena_map & ~(PROG_RANDOM);
            update++;
            if (update > 3) update = 0;
            userconf.rgb_prog_ena_map = update | (userconf.rgb_prog_ena_map & PROG_RANDOM);
            ledprog_rgb_init();
            break;
        }
    }

    save_delay = UI_CONF_SAVE_TIMEOUT;
}



void ui_init()
{
    btn[0].hold = 1200 >> 1;
    btn[0].repeat = 0; // (1000 / 20) >> 1;
    btn[0].cb_push = ui_btn_push_cb;
    btn[0].cb_hold = ui_btn_hold_cb;
    btn[0].cb_release = ui_btn_release_cb;

    btn[1].hold = 1200 >> 1;
    btn[1].repeat = 0;
    btn[1].cb_push = ui_btn_push_cb;
    btn[1].cb_hold = ui_btn_hold_cb;
    btn[1].cb_release = ui_btn_release_cb;
}


void ui_render()
{
    uint8_t w;

    tick++;

    uint8_t prog_pep_idx = userconf.pep_prog_ena_map & ~(PROG_RANDOM);
    uint8_t prog_rgb_idx = userconf.rgb_prog_ena_map & ~(PROG_RANDOM);

    switch (mode) {
        case MODE_RUN: {
            // run programs
            if (ledprog_pep[prog_pep_idx]) {
                ledprog_pep[prog_pep_idx](tick);
            }
            if (ledprog_rgb[prog_rgb_idx]) {
                ledprog_rgb[prog_rgb_idx](tick);
            }

            // check flash save
            if (save_delay) {
                save_delay--;
                if (!save_delay) {
                    userconf_save();
                }
            }

            break;
        }

        case MODE_PROGRAM: {
            break;
        }

        case MODE_PARAMETER: {
            break;
        }
    }

    // set LED current based on ambient light level
    w = adc_get_lsens_coarse();
    w = adc_get_brightness(w);
    is31fl3729_set_global_current(FL3729_ADDR, w);
}
